/*
	Game Categories Light v 1.3
	Copyright (C) 2011, Bubbletune
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <pspsdk.h>
#include <pspkernel.h>
#include <psprtc.h>
#include "psppaf.h"
#include <string.h>
#include <systemctrl.h>
#include "game_categories_light.h"

/* Global variables */
extern char user_buffer[256];
SceUID opened_dfd = -1;
SceUID game_dfd = -1;
int uncategorized;
extern u32 text_addr_game;
int hide_uncategorized;

SceUID sceIoDopenPatched(const char *dirname)
{
	SceUID fd = sceIoDopen(dirname);

	if (sce_paf_private_strcmp(dirname+4, "/PSP/GAME") == 0)
	{
		SceIoStat stat;
		memset(&stat, 0, sizeof(stat));

		if (sceIoGetstat("ef0:/seplugins/hide_uncategorized.txt", &stat) >= 0 ||
			sceIoGetstat("ms0:/seplugins/hide_uncategorized.txt", &stat) >= 0)
		{
			hide_uncategorized = 1;
		}
		
		else
		{
			hide_uncategorized = 0;
		}
		
		ClearCategories();
		uncategorized = 0;
		sce_paf_private_strcpy(user_buffer, dirname);
		game_dfd = fd;
	}

	return fd;
}

int sceIoDreadPatched(SceUID fd, SceIoDirent *dir)
{
	if (fd == game_dfd)
	{
		while (1)
		{
			if (opened_dfd >= 0)
			{
				int res	= sceIoDread(opened_dfd, dir);
				
				if (res > 0)
				{
					if (dir->d_name[0] != '.')
					{
						sce_paf_private_strcpy(dir->d_name+128, dir->d_name);
						sce_paf_private_snprintf(dir->d_name, 128, "%s/%s", user_buffer+14, dir->d_name+128);
						
						if (dir->d_private)
						{
							sce_paf_private_strcpy(dir->d_private+13, dir->d_name);
						}
						
						return res;
					}
					
					else
					{
						continue;
					}
				}
				
				else
				{
					sceIoDclose(opened_dfd);
					opened_dfd = -1;
				}
			}
			
			int res = sceIoDread(fd, dir);
			
			if (res > 0)
			{
				if (dir->d_name[0] != '.')
				{
					if (FIO_S_ISDIR(dir->d_stat.st_mode))
					{
						SceIoStat stat;
						sce_paf_private_memset(&stat, 0, sizeof(SceIoStat));
						sce_paf_private_snprintf(user_buffer+13, 128, "/%s/EBOOT.PBP", dir->d_name);
						
						if (sceIoGetstat(user_buffer, &stat) < 0)
						{
							sce_paf_private_snprintf(user_buffer+13, 128, "/%s/PARAM.PBP", dir->d_name);
							
							if (sceIoGetstat(user_buffer, &stat) < 0)
							{
								u64 mtime;
								sceRtcGetTick((pspTime *)&dir->d_stat.st_mtime, &mtime);
								AddCategory(dir->d_name, mtime);
								
								sce_paf_private_snprintf(user_buffer+13, 128, "/%s", dir->d_name);
								opened_dfd = sceIoDopen(user_buffer);
								
								continue;
							}
			
							else
							{
								if (hide_uncategorized)
								{
									continue; // ignore this Dread
								}
							
								uncategorized = 1;
							}
						}
						
						else
						{
							if (hide_uncategorized)
							{
								continue; // ignore this Dread
							}
						
							uncategorized = 1;
						}
					}
				}
			}
		
			return res;
		}
	}
	
	return sceIoDread(fd, dir);
}

int sceIoDclosePatched(SceUID fd)
{
	if (fd == game_dfd)
	{
		if (uncategorized)
		{
			AddCategory("Uncategorized", 1);
		}
		
		game_dfd = -1;
	}

	return sceIoDclose(fd);
}

int sce_paf_private_snprintf_patched(char *a0, int a1, const char *a2, void *a3, void *t0)
{
	sce_paf_private_strcpy((char *)a1, (char *)t0);
	return sce_paf_private_snprintf(a0, 291, a2, a3, t0);
}

void PatchIoFileMgrForGamePlugin(u32 text_addr)
{
	MAKE_STUB(text_addr+PATCHES->io_dopen_stub, sceIoDopenPatched);
	MAKE_STUB(text_addr+PATCHES->io_dread_stub, sceIoDreadPatched);
	MAKE_STUB(text_addr+PATCHES->io_dclose_stub, sceIoDclosePatched);

	/* SCE renames folders before removal, but it doesn't handle
		categories in doing so. It will try to move things out of
		the category with the rename function... just get rid of
		renaming to fix this lame bug. */
	// #1
	MAKE_CALL(text_addr+PATCHES->snprintf_call_arg_1[0], sce_paf_private_snprintf_patched);
	_sw(0x02402821, text_addr+PATCHES->snprintf_call_arg_1[1]); // li $a1, 291 -> move $a1, $s2
	
	// #2
	MAKE_CALL(text_addr+PATCHES->snprintf_call_arg_2[0], sce_paf_private_snprintf_patched);
	_sw(0x02002821, text_addr+PATCHES->snprintf_call_arg_2[1]); // li $a1, 291 -> move $a1, $s0
}